package com.itbaizhan.controller;


import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.WebResource;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.UUID;

@Controller
public class UploadController {
    @RequestMapping("/fileUpload1")
    public String upload(HttpServletRequest request) throws IOException {
        ServletInputStream is = request.getInputStream();
        int length;
        while ((length=is.read())!=-1){
            System.out.print((char) length);
        }
        return "index";
    }

    @RequestMapping("/fileUpload")
    public String uoload(HttpServletRequest request) throws Exception {
        /* 创建文件夹，存放上传文件 */
        /* 1.设置上传文件的真实路径 */
        String realPath = request.getSession().getServletContext().getRealPath("/upload");
        /* 2.判断该目录是否存在，如果不存在，则创建该目录 */
        File file = new File(realPath);
        if (!file.exists()){
            file.mkdirs();
        }

        /* 分析请求体，找到上传文件数据 */
        /* 1.创建磁盘文件工厂 */
        DiskFileItemFactory factory = new DiskFileItemFactory();
        /* 2.创建上传数据分析器对象 */
        ServletFileUpload servletFileUpload = new ServletFileUpload(factory);
        /* 3.利用分析器对象解析请求体，返回所有数据项 */
        List<FileItem> fileItems = servletFileUpload.parseRequest(request);
        /* 4.遍历所有数据，找到文件项（非表单项） */
        for (FileItem fileItem:fileItems
             ) {
            if (!fileItem.isFormField()){
                /*      将文件数据写入文件夹     */
                /*      1.获取文件名     */
                String name = fileItem.getName();
                /*      2.将文件写入磁盘     */
                fileItem.write(new File(file,name));
                /*      3.删除内存中的临时文件    */
                fileItem.delete();
            }
        }
        return "index";
    }

    /*  ！！注：方法中MultipartFile对象的参数名必须和input:file中的name值相同  */
    @RequestMapping("/fileUpload2")
    public String upload2(MultipartFile file,HttpServletRequest request) throws IOException {
        /* 创建文件夹，存放上传的文件 */
        String realPath = request.getSession().getServletContext().getRealPath("/upload");
        File dir = new File(realPath);
        if (!dir.exists()){
            dir.mkdirs();
        }
        /* 将上传的数据写到文件夹的文件中 */
        /* 1、获得上传文件的文件名 */
        String filename = file.getOriginalFilename();
        filename=UUID.randomUUID()+"_"+filename;
        /* 2、根据文件名在dir目录下创建空文件 */
        File newFile = new File(dir, filename);
        /* 3、将数据写入到文件中 */
        file.transferTo(newFile);
        return "index";
    }

    /* 多文件上传:处理多文件上传,参数类型为MultipartFile数组，参数名和JSP文件控件的name属性一致 */
    @RequestMapping("/fileUpload3")
    public String upload3(MultipartFile[] files,HttpServletRequest request) throws IOException {
        /* 创建文件夹 */
        String realPath = request.getSession().getServletContext().getRealPath("/upload");
        File dir = new File(realPath);
        if (!dir.exists()){
            dir.mkdirs();
        }
        /* 循环文件的数组，将数据一个个放入文件中 */
        for (MultipartFile file:files
             ) {
            String filename = file.getOriginalFilename();
            filename=UUID.randomUUID()+"_"+filename;
            File file1 = new File(dir, filename);
            file.transferTo(file1);
        }

        return "index";
    }

    @RequestMapping("/fileUpload4")
    @ResponseBody  /* 以JSON格式返回数据，并且使页面不跳转 */
    public String upload4(MultipartFile file,HttpServletRequest request) throws IOException {
        String realPath = request.getSession().getServletContext().getRealPath("/upload");
        File dir = new File(realPath);
        if (!dir.exists()){
            dir.mkdirs();
        }

        /* 将数据写入到文件中 */
        String filename = file.getOriginalFilename();
        filename=UUID.randomUUID()+"_"+filename;
        File file1 = new File(dir, filename);
        file.transferTo(file1);
        return "/upload/"+filename;
    }

    @RequestMapping("/fileUpload5")
    @ResponseBody
    public String upload5(MultipartFile file,HttpServletRequest request) throws IOException {
        /* 设置跨服上传路径 */
        String path="http://localhost:8081/upload/";
        /* 获取上传文件的文件名 */
        String filename = file.getOriginalFilename();
        filename=UUID.randomUUID()+"_"+filename;

        /* 跨服上传： */
        /* 1、创建客户端对象 */
        Client client = Client.create();
        /* 2、使用客户端对象连接图片服务器 */
        WebResource resource = client.resource(path + filename);
        /* 3、传输数据 */
        resource.put(file.getBytes());
        return path+filename;
    }

}
